package v1

import (
	"encoding/json"
	"fmt"
	"git.mowie.cc/konrad/Library/models"
	"github.com/labstack/echo"
	"net/http"
	"strconv"
	"strings"
)

type authorPayload struct {
	Author models.Author `json:"author" form:"author"`
}

// AuthorAddOrUpdate is the handler to add or update an author
func AuthorAddOrUpdate(c echo.Context) error {
	// Check for Request Content
	authorFromString := c.FormValue("author")
	var datAuthor models.Author

	if authorFromString == "" {
		b := new(authorPayload)
		if err := c.Bind(b); err != nil {
			fmt.Println(err)
			return c.JSON(http.StatusBadRequest, models.Message{"No author model provided"})
		}
		datAuthor = b.Author
	} else {
		// Decode the JSON
		dec := json.NewDecoder(strings.NewReader(authorFromString))
		err := dec.Decode(&datAuthor)

		if err != nil {
			return c.JSON(http.StatusInternalServerError, models.Message{"Error decoding author: " + err.Error()})
		}
	}

	// Check if we have at least a Lastname
	if datAuthor.Lastname == "" && datAuthor.Forename == "" {
		return c.JSON(http.StatusBadRequest, models.Message{"Please provide at least one name."})
	}

	// Check if we have an ID other than the one in the struct
	id := c.Param("id")
	if id != "" {
		// Make int
		authorID, err := strconv.ParseInt(id, 10, 64)

		if err != nil {
			return c.JSON(http.StatusInternalServerError, models.Message{"Could not get book id"})
		}
		datAuthor.ID = authorID
	}

	// Insert or update the author
	newAuthor, err := models.AddOrUpdateAuthor(datAuthor)

	if err != nil {
		return c.JSON(http.StatusInternalServerError, models.Message{"Error"})
	}

	return c.JSON(http.StatusOK, newAuthor)
}
